package com.alicloud.openservices.tablestore.model.internal;

import com.alicloud.openservices.tablestore.core.utils.Preconditions;
import com.alicloud.openservices.tablestore.core.utils.OptionalValue;

/**
 * 内部接口.
 */
public class TableOptionsEx extends com.alicloud.openservices.tablestore.model.TableOptions {

    private OptionalValue<BloomFilterType> bloomFilterType = new OptionalValue<BloomFilterType>("BloomFilterType");

    private OptionalValue<Integer> blockSize = new OptionalValue<Integer>("BlockSize");

    public TableOptionsEx() {
    }

    public TableOptionsEx(int timeToLive) {
        setTimeToLive(timeToLive);
    }

    public TableOptionsEx(int timeToLive, int maxVersions) {
        setTimeToLive(timeToLive);
        setMaxVersions(maxVersions);
    }

    public TableOptionsEx(int timeToLive, int maxVersions, BloomFilterType bloomFilterType) {
        setTimeToLive(timeToLive);
        setMaxVersions(maxVersions);
        setBloomFilterType(bloomFilterType);
    }

    public TableOptionsEx(int timeToLive, int maxVersions, BloomFilterType bloomFilterType, int blockSize) {
        setTimeToLive(timeToLive);
        setMaxVersions(maxVersions);
        setBloomFilterType(bloomFilterType);
        setBlockSize(blockSize);
    }

    public BloomFilterType getBloomFilterType() {
        if (!bloomFilterType.isValueSet()) {
            throw new IllegalStateException("The value of BloomFilterType is not set.");
        }
        return bloomFilterType.getValue();
    }

    public void setBloomFilterType(BloomFilterType bloomFilterType) {
        this.bloomFilterType.setValue(bloomFilterType);
    }

    public boolean hasSetBloomFilterType() {
        return bloomFilterType.isValueSet();
    }

    public int getBlockSize() {
        if (!blockSize.isValueSet()) {
            throw new IllegalStateException("The value of BlockSize is not set.");
        }
        return blockSize.getValue();
    }

    public void setBlockSize(int blockSize) {
        Preconditions.checkArgument(blockSize > 0, "BlockSize must be greater than 0.");
        this.blockSize.setValue(blockSize);
    }

    public boolean hasSetBlockSize() {
        return blockSize.isValueSet();
    }

    @Override
    public String toString() {
        return super.toString() +
                ", " + bloomFilterType + ", " + blockSize;
    }

    @Override
    protected boolean jsonizeFields(StringBuilder sb, boolean firstItem) {
        firstItem = super.jsonizeFields(sb, firstItem);
        if (this.bloomFilterType.isValueSet()) {
            if (firstItem) {
                firstItem = false;
            } else {
                sb.append(", ");
            }
            sb.append("\"BloomFilterType\": \"");
            sb.append(this.bloomFilterType.getValue());
            sb.append("\"");
        }
        if (this.blockSize.isValueSet()) {
            if (firstItem) {
                firstItem = false;
            } else {
                sb.append(", ");
            }
            sb.append("\"BlockSize\": ");
            sb.append(this.blockSize.getValue());
        }
        return firstItem;
    }

}
